package com.example.base;

import com.example.enums.ResultEnum;
import com.example.util.BeanCopyUtil;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Collection;

/**
 * 控制器基类
 *
 * @author yys
 */
@Component
@SuppressWarnings({"rawtypes", "unchecked"})
public class BaseController {

    protected static final String ACCESS_TOKEN = "accessToken";

    /**
     * 无业务数据成功返回
     *
     * @return
     */
    protected ResultVO success() {
        ResultVO resultVO = new ResultVO();
        resultVO.setData(null);
        return resultVO;
    }

    /**
     * 无业务数据失败返回
     *
     * @return
     */
    protected ResultVO failure() {
        ResultVO resultVO = new ResultVO();
        resultVO.setCode(ResultEnum.FAILURE.getCode());
        resultVO.setMsg(ResultEnum.FAILURE.getMsg());
        resultVO.setData(null);
        return resultVO;
    }

    protected ResultVO failure(String message) {
        ResultVO resultVO = new ResultVO();
        resultVO.setCode(ResultEnum.FAILURE.getCode());
        resultVO.setMsg(message);
        resultVO.setData(null);
        return resultVO;
    }

    /**
     * 有业务数据成功返回
     *
     * @param data
     * @return
     */
    protected ResultVO success(Object data, long count) {
        ResultVO resultVO = new ResultVO();
        resultVO.setData(data);
        resultVO.setCount(count);
        return resultVO;
    }

    /**
     * 有业务数据成功返回
     *
     * @param data
     * @return
     */
    protected ResultVO success(Object data) {
        ResultVO resultVO = new ResultVO();
        resultVO.setData(data);
        return resultVO;
    }

    /**
     * 有业务数据成功返回
     *
     * @param data
     * @return
     */
    protected ResultVO error(String msg) {
        ResultVO resultVO = new ResultVO();
        resultVO.setCode(1);
        resultVO.setData(msg);
        return resultVO;
    }

    /**
     * 操作成功后，返回结果（前后端分离）
     *
     * @param resultVO
     * @return
     */
    protected ResponseBase responseSuccess(ResultVO resultVO) {
        ResponseBase responseBase = new ResponseBase(resultVO.getData());
        RetHead retHead = new RetHead();
        retHead.setErrCode(resultVO.getCode().shortValue());
        retHead.setErrMsg(resultVO.getMsg());
        retHead.setTotal(resultVO.getCount());
        responseBase.setRetHead(retHead);
        return responseBase;
    }


    /**
     * 操作异常后，返回结果（前后端分离）
     *
     * @param resultVO
     * @return
     */
    protected ResponseBase responseError(ResultVO resultVO) {
        RetHead retHead = new RetHead();
        retHead.setErrCode(resultVO.getCode().shortValue());
        retHead.setErrMsg(resultVO.getMsg());
        return new ResponseBase(retHead);
    }

    /**
     * 操作异常后，返回结果（前后端分离）
     *
     * @return
     */
    protected ResponseBase responseError(Integer errCode, String errMsg) {
        RetHead retHead = new RetHead();
        retHead.setErrCode(errCode.shortValue());
        retHead.setErrMsg(errMsg);
        return new ResponseBase(retHead);
    }

    /**
     * 成功返回并转换data对象类型为目标对象类型
     *
     * @param data
     * @param destinationClass 目标Class对象
     * @return
     */
    protected ResultVO success(Object data, Class destinationClass) {
        ResultVO resultVO = new ResultVO();
        if (data != null && data instanceof Collection) {
            Collection collection = (Collection) data;
            resultVO.setData(BeanCopyUtil.convertList(collection, destinationClass));
        } else {
            resultVO.setData(data);
        }
        return resultVO;
    }

    /**
     * 判断调用服务是否成功
     *
     * @param resultVO
     * @return
     */
    public boolean isSuccess(ResultVO resultVO) {
        return ResultEnum.SUCCESS.getCode().equals(resultVO.getCode());
    }

    /**
     * 获取response对象
     *
     * @return
     */
    protected HttpServletResponse getResponseObject() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        return requestAttributes.getResponse();
    }

    /**
     * 获取request对象
     *
     * @return
     */
    protected HttpServletRequest getRequestObject() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        return requestAttributes.getRequest();
    }
}
